/*
 *
 * w_labeler the CMW window labeler user interface
 *
 * I guess we need one of these per window manager...
 */

static char cmw_sccsid[] = "@(#)w_labeler.c 2.7.1.1 95/08/07 SMI; SunOS_CMW";

#include <stdlib.h>
#include <xview/xview.h>
#include <cmw/lbuild.h>
#include <cmw/Xcmw.h>
#include <cmw/priv.h>
#include <cmw/authoriz.h>
#include <stdio.h>
#include <errno.h>


Attr_attribute		INSTANCE;
extern int		_cmw_dont_load_user_defaults;
extern char		*_cmw_supplied_resource_filename;

Display			*dpy;			/* frame's display */
XID			xid;			/* frame's XID */
Attr_attribute		IIL_XID_KEY;

static Xv_object	defsl_lbuild;		/* default SL lbuild handle */
static Bool		outside_ar_is_authorized;
static int		workstation_uid;

static Notify_value	event_checker();	/* looking for our client msg */
static void		render_cmw_lbuild();
static void		render_iil_lbuild();
static void		set_new_iil();
static void		set_new_defsl();

main(argc, argv)
	int		argc;
	char		**argv;
{
	int		i;
	Frame		frame;
	priv_set_t	effective;
	char		*env;
	Rect		rect;

	/*
	 * did we come from the trusted path?
	 */
#ifdef UNIT_TEST
UnitInitProc();
#endif

	if (getprocpriv(sizeof(effective), effective, PRIV_EFFECTIVE)) {
                perror("w_labeler: Cannot get list of effective privileges");
                (void) exit(-1);
        }
	if (!PRIV_ISASSERT(effective, PRIV_SYS_TRUSTED_PATH)) {
		fprintf(stderr, "[%s] can only be run from the trusted path.\n", argv[0]);
		(void) exit(-1);
	}
	if (!PRIV_ISASSERT(effective, PRIV_SYS_TRANS_LABEL))  {
		fprintf(stderr, "[%s] needs 'SYS_TRANS_LABEL' in it's Forced privilege set \n", argv[0]);
		(void) exit(-1);
	}
        if (set_effective_priv(PRIV_ON, 1, PRIV_SYS_TRANS_LABEL))  {
                perror("w_labeler: Cannot turn off effective privileges");
                (void) exit(-1);
        }

	/*
	 * initialize XView
	 */
	_cmw_dont_load_user_defaults = TRUE;

	/*
	 * scan the commandline for any interesting arguments
	 */
	i = 0;
	while (++i < argc) {
		if (!strcmp(argv[i], "-config")) {
			i++;
			_cmw_supplied_resource_filename = argv[i];
		}
	}

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, 0);
	INSTANCE = xv_unique_key();
	IIL_XID_KEY = xv_unique_key();

	rect_construct(&rect, 0, 6969, 64, 64);

	frame = (Frame) xv_create(NULL, FRAME, 
			XV_X, 0,
			XV_Y, 6969,
			FRAME_CLOSED_RECT, &rect,
			NULL);

	dpy = (Display*) xv_get(frame, XV_DISPLAY);
	xid = (XID) xv_get(frame, XV_XID);

	(void) notify_interpose_event_func(frame, event_checker, NOTIFY_SAFE);

	/*
	 * set the current user's environment
	 * ASSUME: env is set correctly
	 */
	env = getenv(WORKSTATION_OWNER);
	if (env) {
		workstation_uid = atoi(env);

		outside_ar_is_authorized = check_authorization(workstation_uid,
					AUTHZ_OUTSIDE_AR, NULL, 0) == 1 ?
			TRUE : FALSE;
	} else {
		fprintf(stderr, "w_labeler: Warning, %s not set.\n",
			WORKSTATION_OWNER);
	}

	/*
	 * initialize the default SL builder in the beginning
	 */
	defsl_lbuild = cmw_lbuild_create(frame, set_new_defsl,
		XV_SHOW, FALSE,
		FRAME_LABEL, "Set Default Sensitivity Label",
		LBUILD_MODE, LBUILD_MODE_SL,
		LBUILD_USERFIELD, "",
		LBUILD_EVENT_MASK, LBUILD_EVENT_APPLY,
		LBUILD_APPLY_ENABLE, FALSE,
		LBUILD_CHECK_AR, outside_ar_is_authorized == 1 ? FALSE: TRUE,
		NULL);

	/*
	 * register this application as the window labeler
	 */
	XCMWregisterTrustedClient(dpy, xid, AGNT_WLABELER);

	xv_main_loop(frame);
}

static Notify_value
event_checker(frame, event, arg, type)
	Frame			frame;
	Event			*event;
	Notify_arg		arg;
	Notify_event_type	type;
{
	/*
	 * check to see if we got one of our special client messages
	 */
	if (event->ie_xevent->type == ClientMessage) {
		static Atom	atom_cmw;
		static Atom	atom_iil;
		static Atom	atom_setlow;
		static Atom	atom_cmwmsg;

		INIT_ATOM(dpy, atom_cmw, WMGR_WLABELER_CMW);
		if (event->ie_xevent->xclient.message_type == atom_cmw) {
			struct wmgr_wlabeler_cmw	*dp = NULL;
			bclabel_t			*proc_cl, *win_cl, clabel;
			bslabel_t			*proc_sl, *win_sl;
			int				trans_label_asserted, priv_error;

			if (XCMWreadTrustedMessage(dpy, event->ie_xevent, &(unsigned char*)dp)) {
				proc_cl = &clabel;
				getcmwplabel(proc_cl);
				proc_sl = bcltosl(proc_cl);
				win_cl = &(dp->window_label);
				win_sl = bcltosl(win_cl);
				trans_label_asserted = FALSE;
				if (! bldominates(proc_sl, win_sl))  {
					priv_error = set_effective_priv(PRIV_ON, 1, PRIV_SYS_TRANS_LABEL);
					if (priv_error)  {
					    perror("w_labeler: Could not assert SYS_TRANS_LABEL privilege");
					} else {
					    trans_label_asserted = TRUE;
					}
				}
				render_cmw_lbuild(frame, dp);
				if (trans_label_asserted)  {
					if (set_effective_priv(PRIV_OFF, 1, PRIV_SYS_TRANS_LABEL)) {
						perror("w_labeler: Could not clear SYS_TRANS_LABEL privilege");
					}
				}
				(void) free(dp);
			}
			return NOTIFY_DONE;
		}

		INIT_ATOM(dpy, atom_iil, WMGR_WLABELER_IIL);
		if (event->ie_xevent->xclient.message_type == atom_iil) {
			struct wmgr_wlabeler_iil	*dp = NULL;
			if (XCMWreadTrustedMessage(dpy, event->ie_xevent, &(unsigned char*)dp)) {
				render_iil_lbuild(frame, dp);
				(void) free(dp);
			}
				
			return NOTIFY_DONE;
		}

		INIT_ATOM(dpy, atom_setlow, MENUMGR_WLABELER_SETLOW);
		if (event->ie_xevent->xclient.message_type == atom_setlow) {
			struct menumgr_wlabeler_setlow	*dp = NULL;
			if (XCMWreadTrustedMessage(dpy, event->ie_xevent, &(unsigned char*)dp)) {
				cmw_lbuild_set(defsl_lbuild,
					LBUILD_VALUE_SL, dp->lower_bound,
					LBUILD_LOWER_BOUND, dp->lower_bound,
					LBUILD_APPLY_ENABLE, TRUE,
					NULL);
				(void) free(dp);
			}

			return NOTIFY_DONE;
		}

		INIT_ATOM(dpy, atom_cmwmsg, WMGR_CMWMSG);
		if (event->ie_xevent->xclient.message_type == atom_cmwmsg) {
			unsigned char			*dp = NULL;
			if (XCMWreadTrustedMessage(dpy, event->ie_xevent, &dp)) {
				cmw_lbuild_set(defsl_lbuild,
					XV_SHOW, TRUE,
					NULL);
				(void) free(dp);
			}

			return NOTIFY_DONE;
		}
	}
	return notify_next_event_func(frame, (Notify_event) event, arg, type);
}

/*
 * put up the window cmw label builder
 */
static void
render_cmw_lbuild(frame, dp)
	Frame				frame;
	struct wmgr_wlabeler_cmw	*dp;
{
	static Xv_object	cmw_lbuild;

	/*
	 * is the lbuild created yet?
	 */
	if (!cmw_lbuild) {
		/* build a new lbuild */
		cmw_lbuild = cmw_lbuild_create(frame, NULL,
			XV_SHOW, FALSE,
			FRAME_LABEL, "Show Full Window Label",
			LBUILD_MODE, LBUILD_MODE_CMW,
			LBUILD_USERFIELD, "Window:",
			LBUILD_APPLY_ENABLE, FALSE,
			LBUILD_CHECK_AR, FALSE,
			LBUILD_EVENT_MASK, 0,
			NULL);
	}

	/*
	 * set in new values
	 */
	cmw_lbuild_set(cmw_lbuild,
			XV_SHOW, TRUE,
			LBUILD_VALUE_CMW, &dp->window_label,
			LBUILD_USERVALUE, dp->window_title,
			NULL);
}
			
/*
 * put up the window iil label builder
 */
static void
render_iil_lbuild(frame, dp)
	Frame				frame;
	struct wmgr_wlabeler_iil	*dp;
{
	static Xv_object	iil_lbuild;

	/*
	 * is the lbuild created yet?
	 */
	if (!iil_lbuild) {
		/* build a new lbuild */
		iil_lbuild = cmw_lbuild_create(frame, set_new_iil,
			XV_SHOW, FALSE,
			FRAME_LABEL, "Set Input Information Label",
			LBUILD_MODE, LBUILD_MODE_IL,
			LBUILD_USERFIELD, "Window:",
			LBUILD_EVENT_MASK, LBUILD_EVENT_APPLY,
			NULL);
	}

	/*
	 * set in new values
	 */
	cmw_lbuild_set(iil_lbuild,
			XV_SHOW, TRUE,
			XV_KEY_DATA, IIL_XID_KEY, dp->window_xid,
			LBUILD_VALUE_IL, &dp->window_iil,
			LBUILD_USERVALUE, dp->window_title,
			LBUILD_UPPER_BOUND, &dp->window_sl,
			NULL);
}

/*
 * the user wants a new window iil
 */
/*ARGSUSED*/
static void
set_new_iil(lbuild, event)
	Xv_object	lbuild;
	LBUILD_EVENT	event;
{
	struct wmgr_wlabeler_set_iil	dp;
	static Atom			atom_set_iil;

	dp.window_xid = (XID) cmw_lbuild_get(lbuild, XV_KEY_DATA, IIL_XID_KEY);
	dp.window_iil = *(bilabel_t*) cmw_lbuild_get(lbuild, LBUILD_VALUE_IL);

	INIT_ATOM(dpy, atom_set_iil, WMGR_WLABELER_SET_IIL);
	
	XCMWsendTrustedMessage(dpy, 
		AGNT_WMGR, 
		-1,
		xid, 
		atom_set_iil, 
		(unsigned char*)&dp, sizeof(dp), 
		atom_set_iil);
}

/*
 * the user wants a new default SL
 */
/*ARGSUSED*/
static void
set_new_defsl(lbuild, event)
	Xv_object	lbuild;
	LBUILD_EVENT	event;
{
	struct wlabeler_menumgr_setsl	dp;
	static Atom			atom_setsl;

	dp.default_sl = *(bslabel_t*) cmw_lbuild_get(lbuild, LBUILD_VALUE_SL);

	INIT_ATOM(dpy, atom_setsl, WLABELER_MENUMGR_SETSL);

	XCMWsendTrustedMessage(dpy,
		AGNT_MENUMGR,
		-1,
		xid,
		atom_setsl,
		(unsigned char*)&dp, sizeof(dp),
		atom_setsl);
}
